// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Collections;
using System.Reflection;
using System.Runtime.Serialization;

using LoCoL.Collections;
using LoCoL.Exceptions;
using LoCoL.Conditions;
using LoCoL.Logs;

namespace LoCoL
{
	/// <summary>
	///  -  .
	/// </summary>
	[Serializable]
	public class Automaton : 
		NamedObject, IValidCheckable, IChangeable, ISerializable
	{

		#region Public Constructor

		/// <summary>
		/// .
		/// </summary>
		/// <param name="id">ID .</param>
		/// <param name="name"> .</param>
		public Automaton( int id, string name )
			: base( id, name )
		{
			m_States = new StateSet();
			m_States.Changed += new ChangedEventHandler( RaiseChangedEvent );
			m_Groups = new StateGroupSet();
			m_Groups.Changed += new ChangedEventHandler( RaiseChangedEvent );
			m_Transitions = new TransitionSet();
			m_Transitions.Changed +=
				new ChangedEventHandler( RaiseChangedEvent );
			m_Transitions.Changed +=
				new ChangedEventHandler( OnTransitionsChanged );
			m_InputVariables = new InputVariableSet();
			m_InputVariables.Changed +=
				new ChangedEventHandler( RaiseChangedEvent );
			m_OutputActions = new OutputActionSet();
			m_OutputActions.Changed +=
				new ChangedEventHandler( RaiseChangedEvent );
			m_AutomatonInputClasses = new ArrayList();
			m_AutomatonInputs = new Hashtable();
			m_AutomatonOutputClasses = new ArrayList();
			m_AutomatonOutputs = new Hashtable();
			m_TransitionsFromState = new Hashtable();
			m_Checkers = new ArrayList();
			Reset();
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected Automaton( SerializationInfo info, StreamingContext context )
			: base( info, context )
		{
			m_States = (StateSet)info.GetValue( "States", typeof(StateSet) );
			m_States.Changed += new ChangedEventHandler( RaiseChangedEvent );
			m_Groups = (StateGroupSet)info.GetValue(
				"Groups", typeof(StateGroupSet) );
			m_Groups.Changed += new ChangedEventHandler( RaiseChangedEvent );
			m_Transitions = (TransitionSet)info.GetValue(
				"Transitions",
				typeof(TransitionSet) );
			m_Transitions.Changed +=
				new ChangedEventHandler( RaiseChangedEvent );
			m_Transitions.Changed +=
				new ChangedEventHandler( OnTransitionsChanged );
			m_InputVariables = (InputVariableSet)info.GetValue(
				"InputVariables",
				typeof(InputVariableSet) );
			m_InputVariables.Changed +=
				new ChangedEventHandler( RaiseChangedEvent );
			m_OutputActions = (OutputActionSet)info.GetValue(
				"OutputActions",
				typeof(OutputActionSet) );
			m_OutputActions.Changed +=
				new ChangedEventHandler( RaiseChangedEvent );
			m_AutomatonInputClasses = new ArrayList();
			m_AutomatonInputs = new Hashtable();
			m_AutomatonOutputClasses = new ArrayList();
			m_AutomatonOutputs = new Hashtable();
			m_TransitionsFromState = new Hashtable();
			m_Checkers = new ArrayList();
			m_InputVariablesResults = new Hashtable();
			Reset();
		}

		/// <summary>
		///    .
		/// </summary>
		public override void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			base.GetObjectData( info, context );
			info.AddValue( "States", m_States );
			info.AddValue( "Groups", m_Groups );
			info.AddValue( "Transitions", m_Transitions );
			info.AddValue( "InputVariables", m_InputVariables );
			info.AddValue( "OutputActions", m_OutputActions );
		}

		#endregion

		#region Private Members

		/// <summary>
		///       .
		/// </summary>
		private Hashtable m_InputVariablesResults;

		/// <summary>
		///     .
		/// </summary>
		private Hashtable m_TransitionsFromState;

		/// <summary>
		/// ID  .
		/// </summary>
		private int m_CurrentStateID;

		/// <summary>
		/// .
		/// </summary>
		private StateSet m_States;

		/// <summary>
		/// .
		/// </summary>
		private TransitionSet m_Transitions;

		/// <summary>
		///  ,   .
		/// </summary>
		private ArrayList m_AutomatonInputClasses;

		/// <summary>
		///   .
		/// </summary>
		private Hashtable m_AutomatonInputs;

		/// <summary>
		///  ,   .
		/// </summary>
		private ArrayList m_AutomatonOutputClasses;

		/// <summary>
		///   .
		/// </summary>
		private Hashtable m_AutomatonOutputs;

		/// <summary>
		///    .
		/// </summary>
		private InputVariableSet m_InputVariables;

		/// <summary>
		///    .
		/// </summary>
		private OutputActionSet m_OutputActions;

		/// <summary>
		///  .
		/// </summary>
		private StateGroupSet m_Groups;

		/// <summary>
		///   ,    .
		/// </summary>
		private AutomatonManager m_Manager;

		/// <summary>
		///  ,  .
		/// </summary>
		private ArrayList m_Checkers;

		#endregion

		#region Private Delegates

		/// <summary>
		///     .
		/// </summary>
		private delegate bool AutomatonInputDelegate();

		/// <summary>
		///     .
		/// </summary>
		private delegate void AutomatonOutputDelegate();

		#endregion

		#region Private Methods

		/// <summary>
		///   ,     .
		/// </summary>
		private void OnTransitionsChanged( object sender, ChangedEventArgs e )
		{
			m_TransitionsFromState.Clear();
		}

		/// <summary>
		///     .
		///      .
		/// </summary>
		/// <param name="manger"> .</param>
		private void EndCycleEventHandler( AutomatonManager manger )
		{
			m_InputVariablesResults.Clear();
		}

		/// <summary>
		///   ,    ,
		///    .
		/// </summary>
		/// <param name="state">
		/// ,    .
		/// </param>
		/// <returns>     .</returns>
		private Transition[] GetTransitionsFromState( State state )
		{
			if ( m_TransitionsFromState[ state.ID ] == null )
			{
				ArrayList array = new ArrayList();
				//   .
				foreach ( Transition t in Transitions )
				{
					//        state,
					//         state.
					if ( !t.IsFromGroup && t.From == state.ID ||
						t.IsFromGroup &&
							Groups[ t.From ].States.Contains( state ) )
					{
						//     
						//     .
						int index = 0;
						while ( array.Count > index &&
							((Transition)array[ index ]).Priority < t.Priority )
						{
							index++;
						}
						array.Insert( index, t );
					}
				}
				//    
				//    -.
				m_TransitionsFromState[ state.ID ] =
					(Transition[])array.ToArray( typeof(Transition) );
			}
			//   .
			return (Transition[])m_TransitionsFromState[ state.ID ];
		}

		#endregion

		#region Internal Methods
		
		/// <summary>
		///     .
		/// </summary>
		internal void Reset()
		{
			m_CurrentStateID = 0;
		}

		/// <summary>
		///  .
		///    .
		/// </summary>
		/// <returns>
		///  (<see cref="Transition"/>),  
		///    <c>null</c>,
		///   .
		/// </returns>
		/// <exception cref="AutomatonException">
		///   .
		/// </exception>
		internal Transition SelectTransition()
		{
			Transition[] trans =
				GetTransitionsFromState( States[ m_CurrentStateID ] );
			Transition t = null;
			for ( int i = 0; i < trans.Length; i++ )
			{
				//       
				if ( t == null )
				{
					//    
					if ( trans[ i ].Condition == null ||
						trans[ i ].Condition.Evaluate( this ) )
					{
						//  
						t = trans[ i ];
					}
					//    
					continue;
				}
				//         
				//   ,   
				if ( trans[ i ].Priority > t.Priority )
				{
					break;
				}
				//    ,
				//     
				//        
				if ( trans[ i ].Condition == null ||
					trans[ i ].Condition.Evaluate( this ) )
				{
					//     
					throw new AutomatonException(
						"  ." );
				}
			}
			return t;
		}

		/// <summary>
		///   .
		/// </summary>
		/// <param name="inputVariableID">ID .</param>
		/// <returns> .</returns>
		internal bool CheckInputVariable( int inputVariableID )
		{
			bool result;
			if ( m_InputVariablesResults[ inputVariableID ] == null )
			{
				InputVariable v = InputVariables[ inputVariableID ];
				Delegate d = (Delegate)m_AutomatonInputs[ inputVariableID ];
				try
				{
					result = (bool)d.DynamicInvoke( null );
				}
				catch ( TargetInvocationException e )
				{
					throw e.InnerException;
				}
				Manager.LogInputInquire( this, v, result );
				m_InputVariablesResults[ inputVariableID ] = result;
			}
			return (bool)m_InputVariablesResults[ inputVariableID ];
		}

		/// <summary>
		///   .
		/// </summary>
		/// <param name="outputActionID">ID .</param>
		internal void DoOutputInvocation( int outputActionID )
		{
			OutputAction a = OutputActions[ outputActionID ];
			Delegate d = (Delegate)m_AutomatonOutputs[ outputActionID ];
			try
			{
				d.DynamicInvoke( null );
			}
			catch ( TargetInvocationException e )
			{
				throw e.InnerException;
			}
			Manager.LogOutputInvocation( this, a );
		}

		/// <summary>
		///  .
		///    .
		/// </summary>
		/// <param name="transition">
		/// ,    .
		/// </param>
		internal void DoTransition( Transition transition )
		{
			State st = States[ transition.To ];
			m_CurrentStateID = transition.To;
			Manager.LogTransition( this, transition );
			transition.Actions.DoActions( this );
			st.Actions.DoActions( this );
		}

		/// <summary>
		///   ,  
		///  .    .
		///  (   Changed)   ,
		///   .
		/// </summary>
		/// <param name="manager"> .</param>
		internal void SetManager( AutomatonManager manager )
		{
			if ( !ReferenceEquals( m_Manager, manager ) )
			{
				if ( m_Manager != null )
				{
					m_Manager.EndCycleEvent -=
						new CycleEventHandler( EndCycleEventHandler );
				}
				m_Manager = manager;
				m_Manager.EndCycleEvent +=
					new CycleEventHandler( EndCycleEventHandler );
				RaiseChangedEvent( this, ChangedEventArgs.Empty );
			}
		}

		#endregion

		#region Overrided Methods

		/// <summary>
		///   .
		/// </summary>
		public override string ToString()
		{
			return "A" + base.ToString();
		}

		#endregion

		#region Public Methods

		/// <summary>
		///  ,   
		///   .
		/// </summary>
		/// <param name="input"> .</param>
		/// <exception cref="AutomatonException">
		/// <list type="bullet">
		/// <item><description>
		///         
		///   InputVariableAttribute.
		/// </description></item>
		/// <item><description>
		///      
		///     .
		/// </description></item>
		/// <item><description>
		///    .
		/// </description></item>
		/// </list>
		/// </exception>
		public void RegisterInput( IAutomatonInput input )
		{
			if ( m_AutomatonInputClasses.Contains( input ) )
			{
				throw new AutomatonException(
					"   ." );
			}
			m_AutomatonInputClasses.Add( input );
			Type type = input.GetType();
			foreach ( MethodInfo mi in type.GetMethods() )
			{
				Attribute[] attr = Attribute.GetCustomAttributes(
					mi,
					typeof(InputVariableAttribute) );
				if ( attr.Length == 0 )
				{
					continue;
				}
				if ( attr.Length > 1 )
				{
					throw new AutomatonException( "  " +
						type.FullName + "." + mi.Name +
						"     " +
						"InputVariableAttribute." );
				}
				int id = ( (InputVariableAttribute)attr[ 0 ] ).ID;
				Delegate d = Delegate.CreateDelegate(
					typeof(AutomatonInputDelegate),
					input,
					mi.Name );
				if ( m_AutomatonInputs[ id ] != null )
				{
					Delegate d0 = (Delegate)m_AutomatonInputs[ id ];
					throw new AutomatonException( " " +
						this.ToString() + "     " + 
						"    : " +
						d0.Method.DeclaringType + "." + d0.Method.Name + "  " +
						d.Method.DeclaringType + "." + d.Method.Name + "." );
				}
				m_AutomatonInputs[ id ] = d;
			}
			RaiseChangedEvent( this, ChangedEventArgs.Empty );
		}

		/// <summary>
		///     ,
		///      .
		/// </summary>
		/// <param name="input"> .</param>
		/// <exception cref="AutomatonException">
		///      .
		/// </exception>
		public void UnregisterInput( IAutomatonInput input )
		{
			if ( !m_AutomatonInputClasses.Contains( input ) )
			{
				throw new AutomatonException(
					"    ." );
			}
			m_AutomatonInputClasses.Remove( input );
			Type type = input.GetType();
			foreach ( MethodInfo mi in type.GetMethods() )
			{
				Attribute[] attr = Attribute.GetCustomAttributes(
					mi,
					typeof(InputVariableAttribute) );
				int id = ( (InputVariableAttribute)attr[ 0 ] ).ID;
				m_AutomatonInputs.Remove( id );
			}
			RaiseChangedEvent( this, ChangedEventArgs.Empty );
		}

		/// <summary>
		///  ,   
		///   .
		/// </summary>
		/// <param name="output"> .</param>
		/// <exception cref="AutomatonException">
		/// <list type="bullet">
		/// <item><description>
		///         
		///   OutputVariableAttribute.
		/// </description></item>
		/// <item><description>
		///         
		///  .
		/// </description></item>
		/// <item><description>
		///    .
		/// </description></item>
		/// </list>
		/// </exception>
		public void RegisterOutput( IAutomatonOutput output )
		{
			if ( m_AutomatonOutputClasses.Contains( output ) )
			{
				throw new AutomatonException(
					"   ." );
			}
			m_AutomatonOutputClasses.Add( output );
			Type type = output.GetType();
			foreach ( MethodInfo mi in type.GetMethods() )
			{
				Attribute[] attr = Attribute.GetCustomAttributes(
					mi,
					typeof(OutputActionAttribute) );
				if ( attr.Length == 0 )
				{
					continue;
				}
				if ( attr.Length > 1 )
				{
					throw new AutomatonException( "  "
						+ type.FullName + "." + mi.Name +
						"     " +
						"OutputVariableAttribute." );
				}
				int id = ( (OutputActionAttribute)attr[ 0 ] ).ID;
				Delegate d = Delegate.CreateDelegate(
					typeof(AutomatonOutputDelegate),
					output,
					mi.Name );
				if ( m_AutomatonOutputs[ id ] != null )
				{
					Delegate d0 = (Delegate)m_AutomatonOutputs[ id ];
					throw new AutomatonException( " " +
						this.ToString() + "     " +
						"    : " +
						d0.Method.DeclaringType + "." + d0.Method.Name + "  " +
						d.Method.DeclaringType + "." + d.Method.Name + "." );
				}
				m_AutomatonOutputs[ id ] = d;
			}
			RaiseChangedEvent( this, ChangedEventArgs.Empty );
		}

		/// <summary>
		///     ,
		///      .
		/// </summary>
		/// <param name="output"> .</param>
		/// <exception cref="AutomatonException">
		///      .
		/// </exception>
		public void UnregisterOutput( IAutomatonOutput output )
		{
			if ( !m_AutomatonOutputClasses.Contains( output ) )
			{
				throw new AutomatonException(
					"    ." );
			}
			m_AutomatonOutputClasses.Remove( output );
			Type type = output.GetType();
			foreach ( MethodInfo mi in type.GetMethods() )
			{
				Attribute[] attr = Attribute.GetCustomAttributes(
					mi,
					typeof(OutputActionAttribute) );
				int id = ( (OutputActionAttribute)attr[ 0 ] ).ID;
				m_AutomatonOutputs.Remove( id );
			}
			RaiseChangedEvent( this, ChangedEventArgs.Empty );
		}


		/// <summary>
		///  ,  .
		/// </summary>
		/// <param name="checker"> .</param>
		/// <exception cref="AutomatonException">
		///    .
		/// </exception>
		public void RegisterChecker( IAutomatonChecker checker )
		{
			if ( m_Checkers.Contains( checker ) )
			{
				throw new AutomatonException( " , " +
					" ,  ." );
			}
			m_Checkers.Add( checker );
		}

		/// <summary>
		///     ,  .
		/// </summary>
		/// <param name="checker"> .</param>
		/// <exception cref="AutomatonException">
		///      .
		/// </exception>
		public void UnregisterChecker( IAutomatonChecker checker )
		{
			if ( !m_Checkers.Contains( checker ) )
			{
				throw new AutomatonException( " , " +
					" ,   ." );
			}
			m_Checkers.Remove( checker );
		}


		#endregion

		#region Public Properties

		/// <summary>
		///  .
		/// </summary>
		public State CurrentState
		{
			get
			{
				return States[ m_CurrentStateID ];
			}
		}

		/// <summary>
		/// .
		/// </summary>
		public StateSet States
		{
			get
			{
				return m_States;
			}
		}

		/// <summary>
		///  .
		/// </summary>
		public StateGroupSet Groups
		{
			get
			{
				return m_Groups;
			}
		}

		/// <summary>
		///    .
		/// </summary>
		public OutputActionSet OutputActions
		{
			get
			{
				return m_OutputActions;
			}
		}

		/// <summary>
		/// .
		/// </summary>
		public TransitionSet Transitions
		{
			get
			{
				return m_Transitions;
			}
		}

		/// <summary>
		///    .
		/// </summary>
		public InputVariableSet InputVariables
		{
			get
			{
				return m_InputVariables;
			}
		}

		/// <summary>
		///   ,    .
		/// </summary>
		public AutomatonManager Manager
		{
			get
			{
				return m_Manager;
			}
		}

		#endregion

		#region IValidCheckable Implementation

		/// <summary>
		///    .
		/// </summary>
		/// <param name="args">      
		///  ,   <see cref="IEnumerable"/>,
		///     ,  
		/// <seealso cref="IAutomatonChecker"/>.</param>
		/// <exception cref="CheckValidException">
		/// <list>
		/// <item><description>
		///      .
		/// </description></item>
		/// <item><description>
		///      ID==0.
		/// </description></item>
		/// <item><description>
		///         ,
		///     .
		/// </description></item>
		/// <item><description>
		///        .
		/// </description></item>
		/// <item><description>
		///        .
		/// </description></item>
		/// <item><description>
		///      IEnumerable,
		///     ,
		///   IAutomatonChecker.
		/// </description></item>
		/// </list>
		/// </exception>
		public void CheckValid( params object[] args )
		{
			//     .
			if ( Manager == null )
			{
				throw new CheckValidException( " " +
					this.ToString() + "     ." );
			}
			//   
			foreach ( Transition t in Transitions )
			{
				t.CheckValid( this );
			}
			//       ID==0.
			if ( States[ 0 ] == null )
			{
				throw new CheckValidException( "  " +
					this.ToString() + "    ID==0.");
			}
			//   
			foreach ( State s in States )
			{
				s.CheckValid( this );
			}
			//    
			foreach ( StateGroup sg in Groups )
			{
				foreach ( State s in sg.States )
				{
					if ( !this.States.Contains( s ) )
					{
						throw new CheckValidException( "  " +
							ToString() + "   " + sg.ToString() +
							"   " + s.ToString() +
							",     .");
					}
				}
			}
			// ,      .
			foreach ( InputVariable iv in InputVariables )
			{
				if ( m_AutomatonInputs[ iv.ID ] == null )
				{
					throw new CheckValidException(
						"      " + 
						iv + "   " + ToString() + "." );
				}
			}
			// ,      .
			foreach ( OutputAction oa in OutputActions )
			{
				if ( m_AutomatonOutputs[ oa.ID ] == null )
				{
					throw new CheckValidException(
						"      " + 
						oa + "   " + ToString() + "." );
				}
			}
			//     ,
			//  IAutomatonChecker.
			foreach ( IAutomatonChecker checker in m_Checkers )
			{
				checker.Check( this );
			}
			//  IAutomatonChecker',   .
			if ( args.Length == 1 && args[ 0 ] != null )
			{
				if ( !( args[ 0 ] is IEnumerable ) )
				{
					throw new CheckValidException( " " +
						this.GetType().Name + "." +
						MethodInfo.GetCurrentMethod().Name +
						"     IEnumerable, " +
						"    ,  " +
						" IAutomatonChecker. ");
				}
				foreach ( object o in (IEnumerable)args[ 0 ] )
				{
					if ( !( o is IAutomatonChecker ) )
					{
						throw new CheckValidException( " " +
							this.GetType().Name + "." +
							MethodInfo.GetCurrentMethod().Name +
							"     IEnumerable," +
							"     , " +
							"  IAutomatonChecker. ");
					}
					((IAutomatonChecker)o).Check( this );
				}
			}
		}

		#endregion

		#region IChangeable Implementaion

		/// <summary>
		/// , ,    .
		/// </summary>
		public event ChangedEventHandler Changed;

		/// <summary>
		///   <see cref="Changed"/>.
		/// </summary>
		private void RaiseChangedEvent( object sender, ChangedEventArgs e )
		{
			if ( Changed != null )
			{
				Changed( this, e );
			}
		}

		#endregion

		#region Operators

		/// <summary>
		///   "<see cref="Automaton"/>==<see cref="State"/>".
		///      <c>a</c>
		///   <c>s</c> .
		/// </summary>
		public static Condition operator ==( Automaton a, State s )
		{
			return new CheckState( a.ID, s.ID );
		}

		/// <summary>
		///   "<see cref="Automaton"/>!=<see cref="State"/>".
		///      <c>a</c>
		///    <c>s</c> .
		/// </summary>
		public static Condition operator !=( Automaton a, State s )
		{
			return !( a == s );
		}

		#endregion

		#region Implicit Conversions

		/// <summary>
		///     <see cref="SendNotificationAction"/>.
		/// ..   <c>e0</c>  <c>automaton</c>.
		/// </summary>
		public static implicit operator SendNotificationAction(
			Automaton automaton )
		{
			return new SendNotificationAction(
				0,
				NotificationType.Event,
				automaton.ID );
		}

		#endregion

	}
}
