// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Reflection;
using System.Runtime.Serialization;

using LoCoL.Exceptions;

namespace LoCoL.Conditions
{
	/// <summary>
	///     .
	/// </summary>
	[Serializable]
	public class CheckState : Condition, ISerializable
	{

		#region Public Constructor

		/// <summary>
		/// .
		/// </summary>
		public CheckState( int automatonID, int stateID )
			: base( automatonID, stateID )
		{
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected CheckState( SerializationInfo info, StreamingContext context )
			: base( info, context, typeof(int), typeof(int) )
		{
		}

		/// <summary>
		///    .
		/// </summary>
		public override void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			base.GetObjectData( info, context );
		}

		#endregion

		#region Overrided Condition Methods

		/// <summary>
		///      .
		/// </summary>
		public override bool Evaluate( Automaton automaton )
		{
			bool result = automaton.Manager.CheckState(
				(int)this[ 0 ],
				(int)this[ 1 ] );
			Automaton checkedAutomaton =
				automaton.Manager.Automatons[ (int)this[ 0 ] ];
			automaton.Manager.LogStateChecking(
				automaton,
				checkedAutomaton,
				checkedAutomaton.States[ (int)this[ 1 ] ],
				result );
			return result;
		}

		#endregion

		#region IValidCheckable Implementation

		/// <summary>
		/// .
		/// </summary>
		/// <exception cref="CheckValidException">
		/// <list>
		/// <item><description>
		///      <see cref="Automaton"/>.
		/// </description></item>
		/// <item><description>
		///        ID.
		/// </description></item>
		/// <item><description>
		///       ID.
		/// </description></item>
		/// </list>
		/// </exception>
		public override void CheckValid( params object[] args )
		{
			if ( args.Length != 1 || !( args[ 0 ] is Automaton ) )
			{
				throw new CheckValidException( " " +
					this.GetType().Name + "." +
					MethodInfo.GetCurrentMethod().Name +
					"     Automaton." );
			}
			Automaton a = (Automaton)args[ 0 ];
			Automaton targetA = a.Manager.Automatons[ (int)this[ 0 ] ];
			if ( targetA.Equals( null ) )
			{
				throw new CheckValidException( "  " +
					"    ID==" + this[ 0 ] + "." );
			}
			if ( targetA.States[ (int)this[ 1 ] ] == null )
			{
				throw new CheckValidException( " " + a.ToString() +
					"     ID==" + this[ 1 ] + "." );
			}
		}

		#endregion

	}
}
