// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Reflection;
using System.Collections;
using System.Runtime.Serialization;

using LoCoL.Exceptions;

namespace LoCoL.Conditions
{

	/// <summary>
	///  .
	/// </summary>
	[Serializable]
	public abstract class Condition :
		IEnumerable, IValidCheckable, ISerializable
	{

		#region Protected Constructor

		/// <summary>
		/// .
		/// </summary>
		protected Condition( params object[] nodes )
		{
			m_Nodes =(object[])nodes.Clone();
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected Condition( SerializationInfo info, StreamingContext context )
		{
			m_Nodes = new object[ info.MemberCount ];
			for ( int i = 0; i < info.MemberCount; i++ )
			{
				m_Nodes[ i ] = info.GetValue( "Node_" + i, typeof(object) );
			}
		}

		/// <summary>
		///      .
		/// </summary>
		protected Condition(
			SerializationInfo info,
			StreamingContext context,
			params Type[] types )
		{
			m_Nodes = new object[ info.MemberCount ];
			for ( int i = 0; i < info.MemberCount; i++ )
			{
				m_Nodes[ i ] = info.GetValue( "Node_" + i, types[ i ] );
			}
		}

		/// <summary>
		///    .
		/// </summary>
		public virtual void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			int i = 0;
			foreach ( object o in m_Nodes )
			{
				info.AddValue( "Node_" + i, o );
				i++;
			}
		}

		#endregion

		#region Private Members

		/// <summary>
		///  .
		/// </summary>
		private object[] m_Nodes;

		#endregion

		#region Public Properties

		/// <summary>
		///      .
		/// </summary>
		/// <exception cref="ConditionException">
		///      .
		/// </exception>
		public object this[ int index ]
		{
			get
			{
				if ( index < 0 || index >= m_Nodes.Length )
				{
					throw new ConditionException(
						"     ." );
				}
				return m_Nodes[ index ];
			}
		}

		#endregion

		#region Public Methods

		/// <summary>
		///      enumerator.
		/// </summary>
		public IEnumerator GetEnumerator()
		{
			return m_Nodes.GetEnumerator();
		}

		/// <summary>
		///      .
		/// </summary>
		public abstract bool Evaluate( Automaton automaton );

		#endregion

		#region Operators

		/// <summary>
		///   AND.
		/// </summary>
		public static Condition operator &( Condition c1, Condition c2 )
		{
			if ( c1 == null )
			{
				return c2;
			}
			if ( c2 == null )
			{
				return c1;
			}
			return new AndCondition( c1, c2 );
		}

		/// <summary>
		///   OR.
		/// </summary>
		public static Condition operator |( Condition c1, Condition c2 )
		{
			if ( c1 == null )
			{
				return c2;
			}
			if ( c2 == null )
			{
				return c1;
			}
			return new OrCondition( c1, c2 );
		}

		/// <summary>
		///   XOR.
		/// </summary>
		public static Condition operator ^( Condition c1, Condition c2 )
		{
			if ( c1 == null )
			{
				return c2;
			}
			if ( c2 == null )
			{
				return c1;
			}
			return new XorCondition( c1, c2 );
		}

		/// <summary>
		///   NOT.
		/// </summary>
		public static Condition operator !( Condition c )
		{
			if ( c == null )
			{
				return c;
			}
			return new NotCondition( c );
		}

		#endregion

		#region IValidCheckable Implementation

		/// <summary>
		///     .
		/// </summary>
		/// <param name="args"> Automaton.</param>
		/// <exception cref="CheckValidException">
		/// <list>
		/// <item><description>
		///      <see cref="Automaton"/>.
		/// </description></item>
		/// <item><description>
		///   
		///       <see cref="CheckValid"/>.
		/// </description></item>
		/// </list>
		/// </exception>
		public virtual void CheckValid( params object[] args )
		{
			if ( args.Length != 1 || !( args[ 0 ] is Automaton ) )
			{
				throw new CheckValidException( " " + 
					this.GetType().Name + "." +
					MethodInfo.GetCurrentMethod().Name + 
					"     Automaton." );
			}
			foreach ( object obj in this )
			{
				if ( obj is IValidCheckable )
				{
					((IValidCheckable)obj).CheckValid( args );
				}
				else
				{
					throw new CheckValidException(
						"   " +
						"      CheckValid().");
				}
			}
		}

		#endregion

	}
}
