// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Collections;
using System.Runtime.Serialization;

using LoCoL.Exceptions;

namespace LoCoL.Collections
{
	/// <summary>
	///   .
	/// </summary>
	[Serializable]
	public class NamedObjectSet :
		IEnumerable, IChangeable, ISerializable, IDeserializationCallback
	{

		#region Public Constructors

		/// <summary>
		/// .
		/// </summary>
		public NamedObjectSet()
		{
			m_Values = new Hashtable();
		}

		/// <summary>
		///  .
		/// </summary>
		public NamedObjectSet( NamedObjectSet namedObjectSet )
		{
			m_Values = (Hashtable)namedObjectSet.m_Values.Clone();
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected NamedObjectSet(
			SerializationInfo info,
			StreamingContext context )
		{
			m_DeserializedObjects = new ArrayList();
			for ( int i = 0; i < info.MemberCount; i++ )
			{
				NamedObject o = (NamedObject)info.GetValue(
					"Value_" + i,
					typeof(NamedObject) );
				m_DeserializedObjects.Add( o );
			}
		}

		/// <summary>
		///    .
		/// </summary>
		public virtual void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			int i = 0;
			foreach ( NamedObject o in m_Values.Values )
			{
				info.AddValue( "Value_" + i, o );
				i++;
			}
		}

		/// <summary>
		///        
		/// ,   .
		/// </summary>
		private ArrayList m_DeserializedObjects;

		#endregion

		#region IDeserializationCallback Implementation

		/// <summary>
		///  .
		/// </summary>
		public void OnDeserialization( object sender )
		{
			m_Values = new Hashtable();
			foreach ( NamedObject o in m_DeserializedObjects )
			{
				this.Add( o );
			}
			m_DeserializedObjects = null;
		}

		#endregion

		#region Private Members

		/// <summary>
		///    .
		/// </summary>
		private Hashtable m_Values;

		#endregion

		#region Public Methods

		/// <summary>
		///  .
		/// </summary>
		public NamedObjectSet Clone()
		{
			return new NamedObjectSet( this );
		}

		/// <summary>
		///  .
		/// </summary>
		public void Clear()
		{
			m_Values.Clear();
			RaiseChangedEvent( this, ChangedEventArgs.Empty );
		}

		/// <summary>
		/// .
		/// </summary>
		/// <exception cref="LoCoLCollectionsException">
		///     .
		/// </exception>
		public void Add( NamedObject namedObject )
		{
			if ( !Contains( namedObject ) )
			{
				m_Values.Add( namedObject.ID, namedObject );
				if ( namedObject is IChangeable )
				{
					((IChangeable)namedObject).Changed +=
						new ChangedEventHandler( RaiseChangedEvent );
				}
				RaiseChangedEvent( this, ChangedEventArgs.Empty );
			}
			else
			{
				throw new LoCoLCollectionsException( " " + namedObject +
					"    ." );
			}
		}

		/// <summary>
		/// ,     .
		/// </summary>
		public bool Contains( NamedObject namedObject )
		{
			return m_Values.ContainsValue( namedObject );
		}

		/// <summary>
		///    .
		/// </summary>
		/// <exception cref="LoCoLCollectionsException">
		///    .
		/// </exception>
		public void Remove( NamedObject namedObject )
		{
			if ( Contains( namedObject ) )
			{
				m_Values.Remove( namedObject.ID );
				if ( namedObject is IChangeable )
				{
					((IChangeable)namedObject).Changed -=
						new ChangedEventHandler( RaiseChangedEvent );
				}
				RaiseChangedEvent( this, ChangedEventArgs.Empty );
			}
			else
			{
				throw new LoCoLCollectionsException( " " + namedObject +
					"   ." );
			}
		}

		#endregion

		#region Public Properties

		/// <summary>
		///     ID.
		/// ID -   ,      .
		/// </summary>
		public NamedObject this[ int id ]
		{
			get
			{
				return (NamedObject)m_Values[ id ];
			}
		}

		/// <summary>
		///    .
		/// </summary>
		public int Count
		{
			get
			{
				return m_Values.Count;
			}
		}

		#endregion

		#region IEnumerable Implementation

		/// <summary>
		/// Enumerator.
		/// </summary>
		public IEnumerator GetEnumerator()
		{
			return m_Values.Values.GetEnumerator();
		}

		#endregion

		#region IChangeable Implementation

		/// <summary>
		/// , ,     .
		/// </summary>
		public event ChangedEventHandler Changed;

		private void RaiseChangedEvent( object sender, ChangedEventArgs e )
		{
			if ( Changed != null )
			{
				Changed( this, e );
			}
		}

		#endregion

	}
}
