// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Runtime.Serialization;

namespace LoCoL
{
	/// <summary>
	/// .
	/// </summary>
	[Serializable]
	public class Notification : NamedObject, ISerializable
	{

		#region Public Constructor

		/// <summary>
		/// .
		/// </summary>
		public Notification( int id, string name )
			: this( id, name, NotificationType.NotSpecified )
		{
		}

		#endregion

		#region Private Constructors

		/// <summary>
		/// .
		/// </summary>
		private Notification(
			int id,
			string name,
			NotificationType notificationType )
			: this(
				id,
				name,
				notificationType,
				new Notification[
					Enum.GetValues( typeof(NotificationType) ).Length ] )
		{
		}

		/// <summary>
		/// .
		/// </summary>
		private Notification(
			int id,
			string name,
			NotificationType notificationType,
			Notification[] allTypes )
			: base( id, name )
		{
			m_Type = notificationType;
			m_AllTypes = allTypes;
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected Notification(
			SerializationInfo info,
			StreamingContext context )
			: base( info, context )
		{
			m_Type = (NotificationType)info.GetValue(
				"NotificationType",
				typeof(NotificationType) );
			m_AllTypes = new Notification[
				Enum.GetValues( typeof(NotificationType) ).Length ];
		}

		/// <summary>
		///    .
		/// </summary>
		public override void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			base.GetObjectData( info, context );
			info.AddValue( "NotificationType", m_Type );
		}

		#endregion

		#region Private Members

		/// <summary>
		///  .
		/// </summary>
		private NotificationType m_Type;

		/// <summary>
		///     .   
		///      
		///   .
		/// </summary>
		private Notification[] m_AllTypes;

		#endregion

		#region Public Properties

		/// <summary>
		///  .
		/// </summary>
		public NotificationType Type
		{
			get
			{
				return m_Type;
			}
		}

		#endregion

		#region Public Methods

		/// <summary>
		///    .
		/// </summary>
		public Notification CreateNotification(
			NotificationType notificationType )
		{
			if ( m_AllTypes[ (int)notificationType ] == null )
			{
				m_AllTypes[ (int)notificationType ] =
					new Notification( this.ID, this.Name, notificationType );
			}
			return m_AllTypes[ (int)notificationType ];
		}

		/// <summary>
		///  .
		/// </summary>
		public Notification CreateMessage()
		{
			return CreateNotification( NotificationType.Message );
		}

		/// <summary>
		///  .
		/// </summary>
		public Notification CreateEvent()
		{
			return CreateNotification( NotificationType.Event );
		}

		#endregion

		#region Overrided Methods

		/// <summary>
		///   .
		/// </summary>
		public override string ToString()
		{
			char c;
			switch ( this.Type )
			{
			case NotificationType.Message:
				c = 'm';
				break;
			case NotificationType.Event:
				c = 'e';
				break;
			default:
				c = 'n';
				break;
			}
			return c + base.ToString();
		}

		#endregion

	}
}
