// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Reflection;
using System.Runtime.Serialization;

using LoCoL.Exceptions;

namespace LoCoL
{
	/// <summary>
	///  :  .
	/// </summary>
	[Serializable]
	public class SendNotificationAction : IAutomatonAction, ISerializable
	{

		#region Public Constructors

		/// <summary>
		/// .
		/// </summary>
		public SendNotificationAction(
			Notification notification,
			int toAutomatonID )
			: this( notification.ID, notification.Type, toAutomatonID )
		{
		}

		/// <summary>
		/// .
		/// </summary>
		/// <exception cref="SendNotificationActionException">
		///    .
		/// </exception>
		public SendNotificationAction(
			int notificationID,
			NotificationType notificationType,
			int toAutomatonID )
		{
			if ( notificationType == NotificationType.NotSpecified )
			{
				throw new SendNotificationActionException(
					"   ." );
			}
			m_NotificationID = notificationID;
			m_NotificationType = notificationType;
			m_ToAutomatonID = toAutomatonID;
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected SendNotificationAction(
			SerializationInfo info,
			StreamingContext context )
		{
			m_NotificationID = (int)info.GetValue(
				"NotificationID",
				typeof(int) );
			m_NotificationType = (NotificationType)info.GetValue(
				"NotificationType",
				typeof(NotificationType) );
			m_ToAutomatonID = (int)info.GetValue(
				"ToAutomatonID",
				typeof(int) );
		}

		/// <summary>
		///    .
		/// </summary>
		public virtual void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			info.AddValue( "NotificationID", m_NotificationID );
			info.AddValue( "NotificationType", m_NotificationType );
			info.AddValue( "ToAutomatonID", m_ToAutomatonID );
		}

		#endregion

		#region Private Members

		/// <summary>
		/// ID  .
		/// </summary>
		private int m_NotificationID;

		/// <summary>
		///   .
		/// </summary>
		private NotificationType m_NotificationType;

		/// <summary>
		/// ID ,   .
		/// </summary>
		private int m_ToAutomatonID;

		#endregion

		#region IAutomatonAction Implementation

		/// <summary>
		///    ( )
		///   <c>automaton</c>.
		/// </summary>
		public void DoAction( Automaton automaton )
		{
			if ( m_NotificationType == NotificationType.Event )
			{
				automaton.Manager.RaiseEvent(
					m_ToAutomatonID,
					m_NotificationID );
			}
			else if ( m_NotificationType == NotificationType.Message )
			{
				automaton.Manager.SendMessage(
					m_ToAutomatonID,
					m_NotificationID );
			}
		}

		#endregion

		#region IValidCheckable Implementation

		/// <summary>
		///     ( ).
		/// </summary>
		/// <param name="args"> Automaton.</param>
		/// <exception cref="CheckValidException">
		/// <list>
		/// <item><description>
		///      <see cref="Automaton"/>.
		/// </description></item>
		/// <item><description>
		///   ,   ,
		///      ID.
		/// </description></item>
		/// </list>
		/// </exception>
		public void CheckValid( params object[] args )
		{
			if ( args.Length != 1 || !( args[ 0 ] is Automaton ) )
			{
				throw new CheckValidException( " " + 
					this.GetType().Name + "." +
					MethodInfo.GetCurrentMethod().Name + 
					"     Automaton." );
			}
			Automaton a = (Automaton)args[ 0 ];
			if ( a.Manager.Notifications[ m_NotificationID ] == null )
			{
				throw new CheckValidException( "  , " +
					"  " + a.ToString() +
					",     ID==" + 
					m_NotificationID.ToString() + "." );
			}
		}

		#endregion

	}
}
