// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.IO;

namespace LoCoL.Logs
{
	/// <summary>
	///       .
	/// </summary>
	public class TextWriteLogger : ILogger
	{

		#region Public Constructor

		/// <summary>
		/// .
		/// </summary>
		public TextWriteLogger( TextWriter writer )
		{
			m_Writer = writer;
			m_CallNesting = -1;
		}

		#endregion

		#region Private Members

		/// <summary>
		/// TextWriter,    .
		/// </summary>
		private TextWriter m_Writer;

		/// <summary>
		///    .
		/// </summary>
		private int m_CallNesting;

		#endregion

		#region Public Properties

		/// <summary>
		/// TextWriter,    .
		/// </summary>
		public TextWriter Writer
		{
			get
			{
				return m_Writer;
			}
		}

		#endregion

		#region ILogger Implementation

		/// <summary>
		///    .
		/// </summary>
		void ILogger.OnBeginCycle( AutomatonManager manager )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledBeginCycle( manager ) )
			{
				return;
			}
			Log( '[',
				"   " + manager.CurrentCycle.ToString() );
		}

		/// <summary>
		///      .
		/// </summary>
		void ILogger.OnBeginNotificationProcessing(
			Automaton automaton,
			Notification notification )
		{
			m_CallNesting++;
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledBeginNotificationProcessing(
					automaton,
					notification ) )
			{
				return;
			}
			Log( '{',
				" " + automaton.ToString() +
				"     " +
				notification.ToString() +
				"   " + automaton.CurrentState.ToString() );
		}

		/// <summary>
		///    .
		/// </summary>
		void ILogger.OnInputInquire(
			Automaton automaton,
			InputVariable variable,
			bool result )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledInputInquire( automaton, variable ) )
			{
				return;
			}
			Log( '<',
				" " + automaton.ToString() +
				"    " + variable.ToString() +
				"   : " + ( result ? "" : "" ) );
		}

		/// <summary>
		///    .
		/// </summary>
		void ILogger.OnNotificationChecking(
			Automaton automaton,
			Notification notification,
			bool result )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledNotificationChecking(
					automaton,
					notification ) )
			{
				return;
			}
			Log( 'N',
				" " + automaton.ToString() +
				"     " + notification.ToString() +
				"   : " + ( result ? "" : "" ) );
		}

		/// <summary>
		///       .
		/// </summary>
		void ILogger.OnStateChecking(
			Automaton automaton,
			Automaton checkedAutomaton,
			State state,
			bool result )
		{
			if ( m_LogDetailer != null &&
				m_LogDetailer.IsEnabledStateChecking(
					automaton,
					checkedAutomaton,
					state ) )
			{
				return;
			}
			Log( 'S',
				" " + automaton.ToString() +
				"    " + checkedAutomaton.ToString() +
				"  " + state.ToString() + 
				"   : " + ( result ? "" : "" ) );
		}

		/// <summary>
		///   .
		/// </summary>
		void ILogger.OnTransition( Automaton automaton, Transition transition )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledTransition( automaton, transition ) )
			{
				return;
			}
			Log( 'T',
				" " + automaton.ToString() +
				"   " + transition.ToString( automaton ) );
		}

		/// <summary>
		///    .
		/// </summary>
		void ILogger.OnOutputInvocation(
			Automaton automaton,
			OutputAction action )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledOutputInvocation( automaton, action ) )
			{
				return;
			}
			Log( '>',
				" " + automaton.ToString() +
				"    " + action.ToString() );
		}

		/// <summary>
		///   .
		/// </summary>
		void ILogger.OnSendMessage(
			Automaton targetAutomaton,
			Notification notification )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledSendMessage(
					targetAutomaton,
					notification ) )
			{
				return;
			}
			Log( '!',
				" " + targetAutomaton.ToString() +
				"    " + notification.ToString() );
		}

		/// <summary>
		///     .
		/// </summary>
		void ILogger.OnEndNotificationProcessing(
			Automaton automaton,
			Notification notification )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledEndNotificationProcessing(
					automaton,
					notification ) )
			{
				m_CallNesting--;
				return;
			}
			Log( '}',
				" " + automaton.ToString() +
				"    " + notification.ToString() +
				"   " + automaton.CurrentState.ToString() );
			m_CallNesting--;
		}

		/// <summary>
		///    .
		/// </summary>
		void ILogger.OnEndCycle( AutomatonManager manager )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledEndCycle( manager ) )
			{
				return;
			}
			Log( ']',
				"   " + manager.CurrentCycle );
		}

		private ILogDetailer m_LogDetailer;
		/// <summary>
		/// ,   .
		/// </summary>
		public ILogDetailer LogDetailer
		{
			get
			{
				return m_LogDetailer;
			}
			set
			{
				m_LogDetailer = value;
			}
		}

		#endregion

		#region Private Methods

		/// <summary>
		///  .
		/// </summary>
		/// <param name="typeOfMessage">  .</param>
		/// <param name="message">.</param>
		private void Log( char typeOfMessage, string message )
		{
			m_Writer.Write( "[" + DateTime.Now.ToLongTimeString() + "] " );
			for ( int i = 0; i < m_CallNesting; i++ )
			{
				m_Writer.Write( '\t' );
			}
			m_Writer.WriteLine( typeOfMessage + " " + message );
			m_Writer.Flush();
		}

		#endregion

	}
}
