// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Reflection;
using System.Runtime.Serialization;

using LoCoL.Conditions;
using LoCoL.Collections;
using LoCoL.Exceptions;

namespace LoCoL
{
	/// <summary>
	/// .
	/// </summary>
	[Serializable]
	public class Transition :
		NamedObject, IValidCheckable, IChangeable, ISerializable
	{

		#region Public Constructors

		/// <summary>
		/// .
		/// </summary>
		public Transition(
			int id,
			string name,
			int from,
			bool isFromGroup,
			int to,
			int priority,
			Condition condition,
			AutomatonActionArray actions )
			: base( id, name )
		{
			m_From = from;
			m_IsFromGroup = isFromGroup;
			m_To = to;
			m_Priority = priority;
			m_Condition = condition;
			m_Actions =
				( actions == null ) ?
				( new AutomatonActionArray() ) :
				( actions.Clone() );
			m_Actions.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		/// <summary>
		/// .
		/// </summary>
		public Transition(
			int id,
			string name,
			int from,
			int to,
			int priority,
			Condition condition,
			AutomatonActionArray actions )
			: this( id, name, from, false, to, priority, condition, actions )
		{
		}

		/// <summary>
		/// .
		/// </summary>
		public Transition(
			int id,
			string name,
			int from,
			int to,
			Condition condition,
			AutomatonActionArray actions )
			: this( id, name, from, false, to, 0, condition, actions )
		{
		}

		/// <summary>
		/// .
		/// </summary>
		public Transition(
			int id,
			string name,
			int from,
			int to,
			Condition condition )
			: this( id, name, from, false, to, 0, condition, null )
		{
		}

		/// <summary>
		/// .
		/// </summary>
		public Transition( int id, string name, int from, int to )
			: this( id, name, from, false, to, 0, null, null )
		{
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected Transition( SerializationInfo info, StreamingContext context )
			: base( info, context )
		{
			m_From = (int)info.GetValue( "From", typeof(int) );
			m_IsFromGroup = (bool)info.GetValue( "IsFromGroup", typeof(bool) );
			m_To = (int)info.GetValue( "To", typeof(int) );
			m_Priority = (int)info.GetValue( "Priority", typeof(int) );
			m_Condition = (Condition)info.GetValue(
				"Condition",
				typeof(Condition) );
			m_Actions = (AutomatonActionArray)info.GetValue(
				"Actions",
				typeof(AutomatonActionArray) );
			m_Actions.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		/// <summary>
		///    .
		/// </summary>
		public override void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			base.GetObjectData( info, context );
			info.AddValue( "From", m_From );
			info.AddValue( "IsFromGroup", m_IsFromGroup );
			info.AddValue( "To", m_To );
			info.AddValue( "Priority", m_Priority );
			info.AddValue( "Condition", m_Condition );
			info.AddValue( "Actions", m_Actions );
		}

		#endregion

		#region Private Members

		/// <summary>
		/// <para>
		///   (  ),   ()
		///  .
		/// </para>
		/// <para>
		///   <see cref="IsFromGroup"/>.
		/// </para>
		/// </summary>
		private int m_From;

		/// <summary>
		///       (  ,
		///   ).
		/// </summary>
		private bool m_IsFromGroup;

		/// <summary>
		///  ,    .
		/// </summary>
		private int m_To;

		/// <summary>
		///  .    0.
		///    .
		/// </summary>
		private int m_Priority;

		/// <summary>
		/// ,   .
		///       .
		///    (..  null),   .
		/// </summary>
		private Condition m_Condition;

		/// <summary>
		/// ,   .
		/// </summary>
		private AutomatonActionArray m_Actions;

		#endregion

		#region Public Properties

		/// <summary>
		/// <para>
		///   (  ),   ()
		///  .
		/// </para>
		/// <para>
		///   <see cref="IsFromGroup"/>.
		/// </para>
		/// </summary>
		public int From
		{
			get
			{
				return m_From;
			}
		}

		/// <summary>
		///       (  ,
		///   ).
		/// </summary>
		public bool IsFromGroup
		{
			get
			{
				return m_IsFromGroup;
			}
		}

		/// <summary>
		///  ,    .
		/// </summary>
		public int To
		{
			get
			{
				return m_To;
			}
		}

		/// <summary>
		///  .    0.
		///    .
		/// </summary>
		/// <exception cref="TransitionException">
		///     .
		/// </exception>
		public int Priority
		{
			get
			{
				return m_Priority;
			}
			set
			{
				if ( value < 0 )
				{
					throw new TransitionException( 
						"    ." );
				}
				m_Priority = value;
				RaiseChangedEvent( this, ChangedEventArgs.Empty );
			}
		}

		/// <summary>
		/// ,   .
		///       .
		///    (..  null),   .
		/// </summary>
		public Condition Condition
		{
			get
			{
				return m_Condition;
			}
			set
			{
				m_Condition = value;
				RaiseChangedEvent( this, ChangedEventArgs.Empty );
			}
		}

		/// <summary>
		/// ,   .
		/// </summary>
		public AutomatonActionArray Actions
		{
			get
			{
				return m_Actions;
			}
		}

		#endregion

		#region Public Methods

		/// <summary>
		///          .
		/// </summary>
		/// <param name="automaton">
		/// ,   .
		/// </param>
		/// <returns>.</returns>
		public string ToString( Automaton automaton )
		{
			string result = base.ToString();
			result += "[ ";
			result +=
				( IsFromGroup ) ?
				( "  " + automaton.Groups[ From ].ToString() ) :
				( " " + automaton.States[ From ].ToString() );
			result += "   " + automaton.States[ To ].ToString() + "]";
			return result;
		}

		#endregion

		#region IValidCheckable Implementation

		/// <summary>
		///    .
		/// </summary>
		/// <param name="args"> <see cref="Automaton"/>.</param>
		/// <exception cref="CheckValidException">
		/// <list>
		/// <item><description>
		///      <see cref="Automaton"/>.
		/// </description></item>
		/// <item><description>
		///     .
		/// </description></item>
		/// <item><description>
		///     .
		/// </description></item>
		/// <item><description>
		///     .
		/// </description></item>
		/// </list>
		/// </exception>
		public void CheckValid( params object[] args )
		{
			if ( args.Length != 1 || !( args[ 0 ] is Automaton ) )
			{
				throw new CheckValidException( " " + 
					this.GetType().Name + "." +
					MethodInfo.GetCurrentMethod().Name + 
					"     Automaton." );
			}
			Automaton a = (Automaton)args[ 0 ];
			if ( IsFromGroup && a.Groups[ From ] == null )
			{
				throw new CheckValidException( " " + ToString() +
					"      ID==" +
					From + "  " + a.ToString() + "." );
			}
			if ( !IsFromGroup && a.States[ From ] == null )
			{
				throw new CheckValidException( " " + ToString() +
					"      ID==" +
					From + "  " + a.ToString() + "." );
			}
			if ( a.States[ To ] == null )
			{
				throw new CheckValidException( " " + ToString() +
					"      ID==" +
					From + "  " + a.ToString() + "." );
			}
			if ( Condition != null )
			{
				Condition.CheckValid( args[ 0 ] );
			}
			foreach ( IAutomatonAction aa in Actions )
			{
				aa.CheckValid( a );
			}
		}

		#endregion

		#region IChangeable Implementation

		/// <summary>
		/// , ,      .
		/// </summary>
		public event ChangedEventHandler Changed;

		/// <summary>
		///   <see cref="Changed"/>.
		/// </summary>
		private void RaiseChangedEvent( object sender, ChangedEventArgs e )
		{
			if ( Changed != null )
			{
				Changed( this, e );
			}
		}

		#endregion

	}
}
